@@ -1,4 +1,3 @@
-/* -*- Mode: C; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
 /* GIO - GLib Input, Output and Streaming Library
  * 
  * Copyright (C) 2006-2007 Red Hat, Inc.
@@ -163,7 +162,7 @@ virtual_node_unref (VirtualNode *node)
     virtual_node_free (node, TRUE);
 }
 
-static VirtualNode *
+static G_GNUC_UNUSED VirtualNode *
 virtual_node_ref (VirtualNode *node)
 {
   g_return_val_if_fail (node != NULL, NULL);
