@@ -1,4 +1,3 @@
-/* -*- Mode: C; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
 /* GIO - GLib Input, Output and Streaming Library
  * 
  * Copyright (C) 2008 Benjamin Otte <otte@gnome.org>
@@ -135,8 +134,6 @@ G_DEFINE_TYPE (GVfsBackendFtp, g_vfs_backend_ftp, G_VFS_TYPE_BACKEND)
 
 #define STATUS_GROUP(status) ((status) / 100)
 
-typedef void (* Ftp550Handler) (FtpConnection *conn, const FtpFile *file);
-
 /*** FTP CONNECTION ***/
 
 struct _FtpConnection
@@ -252,8 +249,8 @@ ftp_connection_set_error_from_response (FtpConnection *conn, guint response)
       case 550: /* Requested action not taken. File unavailable (e.g., file not found, no access). */
 	/* FIXME: This is a lot of different errors. So we have to pretend to 
 	 * be smart here. */
-	code = G_IO_ERROR_FAILED;
-	msg = _("Operation failed");
+	code = G_IO_ERROR_NOT_FOUND;
+	msg = _("File unavailable");
 	break;
       case 451: /* Requested action aborted: local error in processing. */
 	code = G_IO_ERROR_FAILED;
@@ -300,7 +297,6 @@ ftp_connection_set_error_from_response (FtpConnection *conn, guint response)
  * RESPONSE_PASS_300: Don't treat 3XX responses, but return them
  * RESPONSE_PASS_400: Don't treat 4XX responses, but return them
  * RESPONSE_PASS_500: Don't treat 5XX responses, but return them
- * RESPONSE_PASS_550: Don't treat 550 responses, but return them
  * RESPONSE_FAIL_200: Fail on a 2XX response
  */
 
@@ -309,8 +305,7 @@ typedef enum {
   RESPONSE_PASS_300 = (1 << 1),
   RESPONSE_PASS_400 = (1 << 2),
   RESPONSE_PASS_500 = (1 << 3),
-  RESPONSE_PASS_550 = (1 << 4),
-  RESPONSE_FAIL_200 = (1 << 5)
+  RESPONSE_FAIL_200 = (1 << 4)
 } ResponseFlags;
 
 /**
@@ -464,7 +459,7 @@ ftp_connection_receive (FtpConnection *conn,
 	return 0;
 	break;
       case 5:
-	if ((flags & RESPONSE_PASS_500) || (response == 550 && (flags & RESPONSE_PASS_550)))
+	if (flags & RESPONSE_PASS_500)
 	  break;
 	ftp_connection_set_error_from_response (conn, response);
 	return 0;
@@ -568,57 +563,6 @@ ftp_connection_send (FtpConnection *conn,
   return response;
 }
 
-static void
-ftp_connection_check_file (FtpConnection *conn,
-                           const Ftp550Handler *handlers,
-                           const FtpFile *file)
-{
-  while (*handlers && !ftp_connection_in_error (conn))
-    {
-      (*handlers) (conn, file);
-      handlers++;
-    }
-}
-
-static guint
-ftp_connection_send_and_check (FtpConnection *conn,
-                               ResponseFlags flags,
-                               const Ftp550Handler *handlers,
-                               const FtpFile *file,
-                               const char *format,
-                               ...) G_GNUC_PRINTF (5, 6);
-static guint
-ftp_connection_send_and_check (FtpConnection *conn,
-                               ResponseFlags flags,
-                               const Ftp550Handler *handlers,
-                               const FtpFile *file,
-                               const char *format,
-                               ...)
-{
-  va_list varargs;
-  guint response;
-
-  /* check that there's no 550 handling used - don't allow bad use of API */
-  g_return_val_if_fail ((flags & RESPONSE_PASS_550) == 0, 0);
-  g_return_val_if_fail (handlers != NULL, 0);
-  g_return_val_if_fail (file != NULL, 0);
-
-  va_start (varargs, format);
-  response = ftp_connection_sendv (conn,
-                                   flags | RESPONSE_PASS_550,
-                                   format,
-                                   varargs);
-  va_end (varargs);
-  if (response == 550)
-    {
-      ftp_connection_check_file (conn, handlers, file);
-      if (!ftp_connection_in_error (conn))
-          ftp_connection_set_error_from_response (conn, response);
-      response = 0;
-    }
-  return response;
-}
-
 static void
 ftp_connection_parse_features (FtpConnection *conn)
 {
@@ -1516,21 +1460,6 @@ do_unmount (GVfsBackend *   backend,
   g_vfs_job_succeeded (G_VFS_JOB (job));
 }
 
-static void
-error_550_is_directory (FtpConnection *conn, const FtpFile *file)
-{
-  guint response = ftp_connection_send (conn, 
-                                        RESPONSE_PASS_550,
-                                        "CWD %s", file);
-
-  if (STATUS_GROUP (response) == 2)
-    {
-      g_set_error (&conn->error, G_IO_ERROR, 
-                   G_IO_ERROR_IS_DIRECTORY,
-                   _("File is directory"));
-    }
-}
-
 static void
 do_open_for_read (GVfsBackend *backend,
 		  GVfsJobOpenForRead *job,
@@ -1539,7 +1468,6 @@ do_open_for_read (GVfsBackend *backend,
   GVfsBackendFtp *ftp = G_VFS_BACKEND_FTP (backend);
   FtpConnection *conn;
   FtpFile *file;
-  static const Ftp550Handler open_read_handlers[] = { error_550_is_directory, NULL };
 
   conn = g_vfs_backend_ftp_pop_connection (ftp, G_VFS_JOB (job));
   if (!conn)
@@ -1548,11 +1476,9 @@ do_open_for_read (GVfsBackend *backend,
   ftp_connection_ensure_data_connection (conn);
 
   file = ftp_filename_from_gvfs_path (conn, filename);
-  ftp_connection_send_and_check (conn,
-		                 RESPONSE_PASS_100 | RESPONSE_FAIL_200, 
-		                 &open_read_handlers[0],
-		                 file,
-		                 "RETR %s", file);
+  ftp_connection_send (conn,
+		       RESPONSE_PASS_100 | RESPONSE_FAIL_200,
+		       "RETR %s", file);
   g_free (file);
 
   if (ftp_connection_in_error (conn))
