@@ -1,5 +1,3 @@
-/* -*- mode: C; c-file-style: "gnu"; indent-tabs-mode: nil; -*- */
-
 /* GIO - GLib Input, Output and Streaming Library
  * 
  * Copyright (C) 2006-2007 Red Hat, Inc.
@@ -53,9 +51,6 @@ struct _GHalVolumeMonitorClass {
 GType g_hal_volume_monitor_get_type (void) G_GNUC_CONST;
 
 GVolumeMonitor *g_hal_volume_monitor_new                          (void);
-GHalVolume     *g_hal_volume_monitor_lookup_volume_for_mount_path (GHalVolumeMonitor *monitor,
-                                                                   const char        *mount_path);
-
 void            g_hal_volume_monitor_force_update                 (GHalVolumeMonitor *monitor);
 
 void            g_hal_volume_monitor_register                     (GIOModule         *module);
